-- $LastChangedDate: 2008-09-02 20:29:21 -0400 (Tue, 02 Sep 2008) $
-- Math and graphics code.
-- See module.txt for documentation and pitfalls.
-- Notes: Use Vec3() instead of tables (XX=0).

----------
-- math --
----------

-- Axis:
XX = 0  -- IMPORTANT must be ZERO-based to interoperate with C++
YY = 1
ZZ = 2
WW = 3

AXIS_INVALID = sim.Math.AXIS_INVALID()  -- defined as ~0 in C++ which varies by CPU
AXIS_X = 0
AXIS_Y = 1
AXIS_Z = 2

AXIS_PITCH = 0
AXIS_YAW   = 1
AXIS_ROLL  = 2

-- Spherical coordinates:
LAT = 0  -- degree 0..180
LON = 1  -- degree 0..360
ALT = 2  -- meter

-- 2D vector class (index starts as zero).
Vec2_ = {}

function Vec2( x, y )  -- this is a ctor
	local t = { [XX] = x, [YY] = y }
	setmetatable( t, Vec2_ )
	Vec2_.__index = Vec2_
	return t
end

function Vec2_:__tostring()
	return "Vec2: "
        .. tostring(self[XX]) .. ","
        .. tostring(self[YY])
end

-- 3D vector class (index starts as zero).
Vec3_ = {}

function Vec3( x, y, z )  -- this is a ctor
	local t = { [XX] = x, [YY] = y, [ZZ] = z }
	setmetatable( t, Vec3_ )
	Vec3_.__index = Vec3_
	return t
end

function Vec3_:__tostring()
	return "Vec3: "
        .. tostring(self[XX]) .. ","
        .. tostring(self[YY]) .. ","
        .. tostring(self[ZZ])
end

-- RGBA class (index starts as zero).
RR = 0
GG = 1
BB = 2
AA = 3

RGBA_ = {}

function RGBA( r, g, b, a )  -- this is a ctor
	local t = { [RR] = r, [GG] = g, [BB] = b, [AA] = a }
	setmetatable( t, RGBA_ )
	RGBA_.__index = RGBA_
	return t
end

function RGBA_:__tostring()
	return "RGBA: "
        .. tostring(self[RR]) .. ","
        .. tostring(self[GG]) .. ","
        .. tostring(self[BB]) .. ","
        .. tostring(self[AA])
end

-- Radians and degrees:
PI         = 3.14159265358979323846
PI_DOUBLE  = 6.28318530717958647692
PI_HALF    = 1.57079632679489661926  -- pi/2 equivalent to 90'
RADIAN_90  = PI_HALF    -- 90'  in radians (quarter circle)
RADIAN_180 = PI         -- 180' in radians (half circle)
RADIAN_360 = PI_DOUBLE  -- 360' in radians (full circle)

-- Math functions:
Deg2Rad = math.rad  -- function

function Distance( v )
	return math.sqrt( v[XX]*v[XX] + v[YY]*v[YY] + v[ZZ]*v[ZZ] )
end
