-- $LastChangedDate: 2008-09-02 20:29:21 -0400 (Tue, 02 Sep 2008) $
-- Updates HUD every tick.
-- See module.txt for documentation and pitfalls.

Hud = {}  -- singleton

-- Initialize HUD.
function Hud.Init()
	Hud.updateTime = 0  -- milliseconds

	-- HUD is composed of lines of text ("HUD strings") defined by HudStringArgs.
	hudStringArgs = {
		text="",
		orientation="BOTTOM_LEFT",
		fontSize = 28.0,
		x = 16,
		y = 16,
	}

	local alpha = 1.0
	local yi = 32

	Hud.hudString1 = sim.HudString.new( hudStringArgs )
	hudStringArgs.y = hudStringArgs.y + yi

	Hud.hudString2 = sim.HudString.new( hudStringArgs )
	hudStringArgs.y = hudStringArgs.y + yi

	Hud.hudString3 = sim.HudString.new( hudStringArgs )
	hudStringArgs.y = hudStringArgs.y + yi

	Hud.hudString4 = sim.HudString.new( hudStringArgs )
	hudStringArgs.y = hudStringArgs.y + yi

	Hud.hudString5 = sim.HudString.new( hudStringArgs )
	Hud.hudString5:SetColor( 1.0, 1.0, 0.0, alpha )  -- yellow
	hudStringArgs.y = hudStringArgs.y + yi

	Hud.hudString6 = sim.HudString.new( hudStringArgs )
	hudStringArgs.y = hudStringArgs.y + yi

	Hud.hudStringOver = sim.HudString.new( { text="", orientation="TOP_LEFT", x=0, y=yi, fontSize = 34.0, } )
	Hud.hudStringOver:SetColor( 1.0, 0.6, 0.6, 1.0 )
end
Hud.Init()  -- do it

-- Update HUD.
function Hud.Update()

	-- Don't update HUD too fast.  Give time for the user to read it.
	if ( sim.Program.GetElapsedTime() - Hud.updateTime < defs.HUD_UPDATE_FREQ ) then
		return
	end
	Hud.updateTime = sim.Program.GetElapsedTime()

	local temp = ""
	local current = Player.aircraft
	local craftPhysics = current:GetCraftPhysics()
	local posSph = sim.World:WorldVertex2SphereVertex( current:GetPosition() )

	-- Line 1.
	if ( current:IfCrash() ) then
		temp = ""
	else
		local speedKPH = sim.Physics:Speed2KPH( craftPhysics:ComputeSpeed() )
		local throttle = current:GetThrottle()
		local compressionScale = 1.0 / sim.World:GetCompressionScale() + 0.01
		local paused = ""
		if ( sim.Program:IfPaused() ) then paused = "[PAUSED]" end
		temp = string.format( "KPH: %3d THROT: %3d%% FPS: %u  [%dX] %s",
							  speedKPH,
						      throttle * 100.0,
						      sim.Window:GetFPS(),
						      compressionScale,
						      paused )
	end
	Hud.hudString1:SetText( temp )

	-- Line 2.
	if ( current:IfCrash() ) then
		temp = ""
	else
		temp = string.format( "LAT,LON,ALT: %.2f %.2f %.2f",
		                      posSph[LAT],
						      posSph[LON],
						      posSph[ALT] )
	end
	Hud.hudString2:SetText( temp )

	-- Line 3.
	if ( current:IfCrash() ) then
		temp = ""
	else
		local coturn = ""
		if ( current:IfCoordinatedTurnExecuting() ) then coturn = "  COTURN  " end
		temp = string.format( "DRAG  : %8.2fN WEIGHT: %8.2fN  AoA: %2d %s",
		                      Distance(craftPhysics:ComputeDragForce()),
						      Distance(craftPhysics:ComputeWeightForce()),
						      craftPhysics:ComputeAngleOfAttack(),
						      coturn )
	end
	Hud.hudString3:SetText( temp )

	-- Line 4.
	if ( current:IfCrash() ) then
		temp = ""
	else
		temp = string.format( "THRUST: %8.2fN LIFT  : %8.2fN",
						      Distance(craftPhysics:ComputeThrustForce()),
						      Distance(craftPhysics:ComputeLiftForce()) )
	end
	Hud.hudString4:SetText( temp )

	-- Line 5.
	if ( current:IfCrash() ) then
		temp = ""
	else
		temp = ""

		if ( current:IfWheelBrakes() ) then
			temp = "WHEEL BRAKES"
		elseif ( current:IfBrakes() ) then
			temp = "AIR BRAKES" 
		end

		if ( craftPhysics:IfStall() ) then
			temp = CatWords( temp, "STALL" )
		end

		local replay = Game.GetPlayerReplay()
		if ( replay:IfPlaying() ) then
			temp = CatWords( temp, "REPLAY" )
		elseif ( replay:IfRecording() ) then
			temp = CatWords( temp, "RECORDING REPLAY" )
		end
	end
	Hud.hudString5:SetText( temp )

	-- Line 6.
	if ( sim.argv2["-jp"] ) then
		temp = string.format( "JOY %05d %05d %05d %05d",
							 sim.Joystick:GetAxisValue( 0 ),
							 sim.Joystick:GetAxisValue( 1 ),
							 sim.Joystick:GetAxisValue( 2 ),
							 sim.Joystick:GetAxisValue( 3 ) )
		Hud.hudString6:SetText( temp )
	end

	-- The "game over" hudstring.
	temp = ""
	if ( current:IfCrash() ) then
		local causeOfCrash = current:GetCauseOfCrash()
		if ( causeOfCrash == sim.Craft.eCauseOfCrash_DOWNWARD_TOO_FAST ) then
			temp = CatWords( temp, "** GAME OVER ** [YOU BURIED YOURSELF]" )
		else
			temp = CatWords( temp, "** GAME OVER ** [INSERT QUARTER]" )
		end
	end
	Hud.hudStringOver:SetText( temp )

end

-- Update the printing of aircraft's name at its 2D projected position.
function Hud.UpdateAircraftText( hudString, aircraft, aircraftType )
	local wv = aircraft:GetPosition()
	local ev = TransformWorld2Eye( wv )
	if ( sim.Math:IF_Z_FACING( ev[ZZ] ) ) then
		-- Facing viewpoint.
		hudString:SetText( sim.Aircraft.names[aircraftType] )
		hudString:SetPosition3D( wv )
	else
		-- Behind viewpoint (culled).
		hudString:SetText( "" )
	end
end
