-- $LastChangedDate: 2008-09-04 10:49:45 -0400 (Thu, 04 Sep 2008) $
-- Mission in OSSIM.
-- mission.lua defines an equivalent of a Mission base class which this extends.
-- See module.txt for documentation and pitfalls.

-- Specialize mission class.
assert( Mission )  -- mission.lua defines base methods
Mission.BaseReset  = Mission.Reset
Mission.placeNames = {
	["Nepal"]           = Vec3( 27.0,   82.0, 4000.0 ),
	["Himalayas"]       = Vec3( 33.2,   78.1, 9000.0 ),
	["Oregon"]          = Vec3( 43.4, -121.7, 1700.0 ),
	["Sahara"]          = Vec3( 22.1,   15.3, 1500.0 ),
	["San Francisco"]   = Vec3( 37.5, -122.4, 1000.0 ),
	["New York"]        = Vec3( 40.3,  -74.0,  500.0 ),
	["Chicago"]         = Vec3( 41.5,  -87.3,  500.0 ),
	["Grand Canyon"]    = Vec3( 36.1, -112.1, 3500.0 ),
	["Florida"]         = Vec3( 27.1,  -82.0,  200.0 ),
	["Antarctica"]      = Vec3( -88,    5.0,  5000.0 ),
}
Mission.initialPositionName = "Nepal"
Mission.light = {
	pos      = Vec3( 2.0, 2.0, -2.0 ),
	ambient  = RGBA(0.50, 0.50, 0.50, 1.00),
	diffuse  = RGBA(0.50, 0.50, 0.50, 1.00),
	specular = RGBA(0.30, 0.30, 0.30, 1.00),
}

-- Reset/init mission.
function Mission.Reset()
	local self = Mission

	-- Call base Reset().
	self.BaseReset()

	-- Using the GUI, the user can select the location by name or by entering lat,lon,alt.
	-- Normally, (lat,lon,alt) will be empty strings.
	local cmdFields = CommandQueue.GetCmdFields()
	if ( cmdFields and (cmdFields[CommandQueue.COMMAND_FIELD] == "RESTART") ) then
		local locationName = cmdFields[CommandQueue.RESTART_FIELD_LOCATION]
		local locationLat  = cmdFields[CommandQueue.RESTART_FIELD_LAT]
		local locationLon  = cmdFields[CommandQueue.RESTART_FIELD_LON]
		local locationAlt  = cmdFields[CommandQueue.RESTART_FIELD_ALT]
		-- Change location (OSSIM).
		if ( locationLat ~= "" and locationLon ~= "" and locationAlt ~= "" ) then
			local pos = Vec3( tonumber(locationLat),
							  tonumber(locationLon),
							  tonumber(locationAlt) )
			self.placeNames["UserDefined"] = pos
			self.initialPositionName = "UserDefined"
			print( "Changing location to (" .. locationLat .. "," .. locationLon .. "," .. locationAlt .. ")." )
		else
			self.initialPositionName = locationName
			print( "Changing location to " .. tostring(locationName) .. "." )
		end
	end

	-- Set initial position of player.
	local initialPosSphere = self.placeNames[self.initialPositionName]
	self.initialPos = sim.World.SphereVertex2WorldVertex( initialPosSphere )

	BeginPlacement( Player.aircraft )  -- disable collision
	Player.aircraft:SetPosition( self.initialPos )
	EndPlacement( Player.aircraft )  -- enable collision

	-- Reset player's aircraft (throttle, speed, etc).
	Player.aircraft:SetThrottle( 0.3 )
	Player.aircraft:SetLandingGear( false )
	sim.World.RotateTangential( Player.aircraft, initialPosSphere )  -- align with surface of planet
	local craftPhysics = Player.aircraft:GetCraftPhysics()
	craftPhysics:SetSpeed( sim.Physics.KPH2Speed( 125.0 ) )

	-- Reset views.
	ChaseView.chaseViewpoint = ChaseView.chaseViewpointPlayer
	if ( not self.ifDoneResetView ) then
		self.ifDoneResetView = true
		ChaseView.chaseViewpoint:Save()
		MakeSecondView()
	end
	sim.SuperView.SetViewMode( sim.SuperView.eViewMode_OTW_FRONT )

	-- Initialize sky and lights.
	sim.Sky:SetLightColor( self.light.ambient, self.light.diffuse, self.light.specular )
	sim.Sky:SetLightPosition( self.light.pos )
end

-- Initialization/start mission (done once).
function Mission.Start()
	local self = Mission

	-- Initial reset.
	self.Reset()
end

-- Mission-specific timer-tick handler.
function Mission.Tick()
	local self = Mission

	-- Process commands from the simulator's "command queue".
	local cmd = sim.Program:DequeueCommand()
	if ( cmd ) then
		if ( self.placeNames[cmd] ) then
			self.initialPositionName = cmd
			self.Reset()
		end
	end

end

-- Let startup.lua continue...

