-- $LastChangedDate: 2008-09-02 20:29:21 -0400 (Tue, 02 Sep 2008) $
-- Scene class.
-- See module.txt for documentation and pitfalls.

-----------
-- Scene --
-----------

Scene = {}

-- Map location names to scene scripts.
Scene.scriptLocationMap = {
	["Golden Gate"] = "scene_GoldenGate.lua",
}

-- (private)
function Scene.CopyObject( t )
	local t2      = {}
	t2.pos        = t.pos
	t2.file       = t.file
	t2.collidable = t.collidable
	t2.scale      = t.scale
	t2.Rotate     = t.Rotate
	return t2
end

-- (private)
function Scene.LoadObjects()
	for k,t in pairs(Scene.objects) do
		if ( not t.scale ) then t.scale = 1.0 end
		t.obj = sim.Object.new( t.pos, t.file, t.scale, t.load )
		t.Rotate( t.obj )
		t.obj:SetCollidable( t.collidable )
		sim.World.AttachObject( t.obj )
	end
end

-- Load scene according to location name.
function Scene.Load( locationName )
	Scene.locationName = locationName  -- remember for Reload()
	local script = Scene.scriptLocationMap[locationName]
	if ( script ) then
		dofile( sim.Program.GetScriptDir() .. "/" .. script )
		Scene.LoadObjects()
		return true
	else
		return false
	end
end

-- The ability to dynamically reload a scene is useful
-- for placing objects while editing a scene script.
function Scene.Reload()
	Log( "Reloading scene." )

	-- If this is a reload, detach old Objects.
	if ( Scene.objects ) then
		for k,t in pairs(Scene.objects) do
			sim.World:DetachObject( t.obj )
		end
	end

	-- Reload scene using saved location name.
	Scene.Load( Scene.locationName )
end
