-- $LastChangedDate: 2008-09-04 10:49:45 -0400 (Thu, 04 Sep 2008) $
-- Called every timer-tick.
-- See module.txt for documentation and pitfalls.

-- NOP if controlled by trackball.
if ( sim.argv2["-t"] ) then
	return
end

-- Keyboard handler.
keyEvent = sim.Keyboard.DequeueKey()
if ( keyEvent ) then
	Keyboard.Handler( keyEvent )
end

-- Joystick handler (button).
joyButtonEvent = sim.Joystick.DequeueButton()
if ( joyButtonEvent ) then
	if ( joyButtonEvent.buttonVal ) then
		-- Button press event.
		-- Let any button control brakes (interim).
		Player.aircraft:ToggleBrakes()
	else
		-- Button release event.
	end
end

-- Update HUD.
Hud.Update()

-- Update engine sound.
engineVolume = Player.aircraft:GetThrottle()  -- throttle equals volume {0.0,..,1.0}
if ( engineVolume < 0.05 or Player.aircraft:IfCrash() ) then
	Player.aircraft:StopEngineSound()
else
	Player.aircraft:PlayEngineSound( engineVolume )
end

-- Update stall warning sound.
if ( Player.aircraft:GetCraftPhysics():IfStall() ) then
	sim.Sound.Play( "stall.wav.gz", sim.Sound.PLAY_LOOP, 0.6 )
else
	sim.Sound.Stop( "stall.wav.gz" )
end

-- Cut-scene: death roll with rescue helicopter.
if ( Player.aircraft:IfCrash() ) then

	-- This updates only on state transition.
	Game.UpdateScore( Game.SCORE_CRASH )

	-- Show rescue helicopter.
	RescueHelicopter.Start()

	-- Rotate slowly (player's chase-plane).
	ChaseView.chaseViewpointPlayer:Rotate( sim.ChaseViewpoint.AXIS_CHASE_PLANE_HORZ,
				                           -Keyboard.ChaseViewRotation() * 0.5 )
end

-- Process commands from the simulator's "command queue".
CommandQueue.Process()  -- cmd.lua

-- CRITICAL: Collect garbage to induce freeing memory of 3D models
-- (which consume large amounts of memory).
if ( (not gGarbageCollectionTime) or (sim.Program.GetElapsedTime() - gGarbageCollectionTime > 500) ) then
	gGarbageCollectionTime = sim.Program.GetElapsedTime()  -- milliseconds
	collectgarbage()
end

-- Call mission-specific timer-tick handler.
Mission.Tick()
