/*****************************************************************************
 * $LastChangedDate: 2008-06-13 22:41:29 -0400 (Fri, 13 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Interface of C functions for Lua scripts.
 * @remarks module.txt documents how to export new C++ classes to Lua.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "fx/module.hh"
#include "fx/motion_blur.hh"
using namespace fx;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "FX"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * Window:EnableMotionBlur( persistence )
 *****************************************************************************/
INTERN int
FX_EnableMotionBlur( lua_State* L )
{
	try
	{
		const fp persistence = POP_FP( L );
		MotionBlur::GetInstance().Enable( persistence );
	}
	CATCH_LUA_EXCEPTION
	return 0;  // number of pushed values
}

/*****************************************************************************
 * Window:DisableMotionBlur()
 *****************************************************************************/
INTERN int
FX_DisableMotionBlur( lua_State* L )
{
	try
	{
		MotionBlur::GetInstance().Disable();
	}
	CATCH_LUA_EXCEPTION
	return 0;  // number of pushed values
}

//==============================================================================
// Register C functions with Lua.
//==============================================================================

#if ! DOXYGEN

INTERN const struct luaL_reg sMethodTableNew[] =
{
	// (new and __gc omitted)
	{	NULL,			NULL		}
};
    
INTERN const struct luaL_reg sMethodTable[] =
{
	{	"EnableMotionBlur",			FX_EnableMotionBlur },
	{	"DisableMotionBlur",		FX_DisableMotionBlur },
	{	NULL,						NULL }
};

void
RegisterFX( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN

} // namespace lua_bind
