/*****************************************************************************
 * $LastChangedDate: 2008-06-28 20:22:40 -0400 (Sat, 28 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Interface of C functions for Lua scripts.
 * @remarks module.txt documents how to export new C++ classes to Lua.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#if COMPILE_GUI

#include "base/module.hh"
using namespace base;
#include "gui/module.hh"
using namespace gui;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Gui"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * Gui:Show()
 *****************************************************************************/
INTERN int
Gui_Show( lua_State* L )
{
	GET_GUI().Show();
	return 0;  // number of pushed values
}

//==============================================================================
// Register C functions with Lua.
//==============================================================================

#if ! DOXYGEN

INTERN const struct luaL_reg sMethodTableNew[] =
{
	// (new and __gc omitted)
	{	NULL,			NULL		}
};
    
INTERN const struct luaL_reg sMethodTable[] =
{
	{	"Show",					Gui_Show },
	{	NULL,					NULL }
};

void
RegisterGui( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN

} // namespace lua_bind

#endif // COMPILE_GUI
