/*****************************************************************************
 * $LastChangedDate: 2008-06-24 23:07:05 -0400 (Tue, 24 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Interface of C functions for Lua scripts.
 * @remarks module.txt documents how to export new C++ classes to Lua.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "sky/module.hh"
using namespace sky;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Sky"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * Sky:SetLightPosition( Vec3 )
 *****************************************************************************/
INTERN int
Sky_SetLightPosition( lua_State* L )
{
	try
	{
		const WorldVertex pos = POP_VECTOR3<WorldVertex>( L );

		GET_SKY().GetLight()->SetPosition(pos);
	}
	CATCH_LUA_EXCEPTION
	return 0;  // number of pushed values
}

/*****************************************************************************
 * Sky:SetLightColor( ambient:RGBA, diffuse:RGBA, specular:RGBA )
 *****************************************************************************/
INTERN int
Sky_SetLightColor( lua_State* L )
{
	try
	{
		const RGBA specular( POP_RGBA( L ) );
		const RGBA diffuse(  POP_RGBA( L ) );
		const RGBA ambient(  POP_RGBA( L ) );

		GET_SKY().GetLight()->SetAmbient( ambient );
		GET_SKY().GetLight()->SetDiffuse( diffuse );
		GET_SKY().GetLight()->SetSpecular( specular );
	}
	CATCH_LUA_EXCEPTION
	return 0;  // number of pushed values
}

//==============================================================================
// Register C functions with Lua.
//==============================================================================

#if ! DOXYGEN

INTERN const struct luaL_reg sMethodTableNew[] =
{
	// (new and __gc omitted)
	{	NULL,			NULL		}
};
    
INTERN const struct luaL_reg sMethodTable[] =
{
	{	"SetLightPosition",		Sky_SetLightPosition },
	{	"SetLightColor",		Sky_SetLightColor },
	{	NULL,					NULL }
};

void
RegisterSky( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN

} // namespace lua_bind
