/*****************************************************************************
 * $LastChangedDate: 2008-06-24 23:07:05 -0400 (Tue, 24 Jun 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Misc functions.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define PROGRAM_FUNCS_CC 1
#include "base/module.hh"
using namespace base;
#include "math/module.hh"
using namespace math;
#include "graph/module.hh"
#include "graph/model_cache.hh"
#include "graph/light.hh"
using namespace graph;
#include "object/module.hh"
using namespace object;
#include "sky/module.hh"
using namespace sky;
#include "program/module.hh"
#include "program/funcs.hh"

namespace program {

/*****************************************************************************
 * Load a 3D model according to CraftSpecs.
 * @param	craftSpecs
 * @param	loadCopy
 *          If false, return a possibly shared 3D model.
 *          If true, return an independent copy.
 *          For dynamic aircraft, nodes should be copies so that propellers etc
 *          can be rotated independently without affecting other aircraft.
 *****************************************************************************/
shptr<Graph>
LoadModel( const CraftSpecs& craftSpecs, const bool loadCopy )
{
ASSERT( not craftSpecs.mModelFile.empty() );
ASSERT( craftSpecs.mLength > Meter(0) );
ASSERT( craftSpecs.mModelScale > 0 );

	// Load 3D model.
	shptr<Graph> graph = \
	ModelCache::GetInstance().LoadModel( craftSpecs.mModelFile,
                                         craftSpecs.mLength,		// meters
                                         craftSpecs.mModelOffset,	// meters
                                         craftSpecs.mModelRotationList,
                                         craftSpecs.mModelScale,
                                         loadCopy );
	return graph;
}

/*****************************************************************************
 * Insert light source into a Graph.
 *****************************************************************************/
shptr<Light>
InsertLightIntoGraph( shptr<Graph> graph,
                      const uint lightNum,
                      const RGBA ambient,
                      const RGBA diffuse,
                      const RGBA specular )
{
CHECK_TYPESIG(graph,TYPESIG_GRAPH);

	shptr<Light> light = new Light( lightNum,
	                                GET_SKY().GetLight()->GetPosition(),
                                    ambient,
                                    diffuse,
                                    specular );
	graph->InsertGroupNode( light->GetGroupNode() );
	return light;
}

} // namespace program
