/*****************************************************************************
 * $LastChangedDate: 2008-08-31 02:29:07 -0400 (Sun, 31 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Updates HUD every tick.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef PROGRAM_HUD_HH
#define PROGRAM_HUD_HH 1

#include "hud/module.hh"
using namespace hud;

namespace program {

////////////////////////////////////////////////////////////////////////////////
/// @brief Singleton to update HUD.  Contains HudStrings.
///
/// To activate HudUpdater, call Start().
///
class HudUpdater
{
PREVENT_COPYING(HudUpdater)
private:
					HudUpdater( void );
					~HudUpdater();

public:
	CLASS_METHOD HudUpdater& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new HudUpdater;
		return *msInstance;
	}

public:
	void			Start( void );

private:
	shptr<Hud::HudString>	CtorHudStringSplash( void );
	void					UpdateFast( const Milliseconds millisecElapsed );

private:
	/// @brief Timer-tick.
	///
	class TickFunctor : public base::Timer::Functor
	{
	public:
		void	operator()( const Milliseconds millisecElapsed );
	};
	friend class TickFunctor;

private:
	// Each HudString represents a line of text in HUD.
	CLASS_VAR HudUpdater*	msInstance;
	Hud&					mHud;
	shptr<Hud::HudString>	mHudStringSplash;
};

} // namespace program

#endif // PROGRAM_HUD_HH
