/*****************************************************************************
 * $LastChangedDate: 2008-08-31 19:34:22 -0400 (Sun, 31 Aug 2008) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	World class.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_WORLD_HH
#define WORLD_WORLD_HH 1

#include "base/module.hh"
#include "base/timer.hh"
using namespace base;
#include "graph/module.hh"
using namespace graph;
#include "object/module.hh"
using namespace object;
#include "world/state_sorter.hh"

namespace world {

#define GET_WORLD() (gWorld)	// world/globals.hh, gWorld is a macro
#define ASSERT_WORLD()			// NOP

////////////////////////////////////////////////////////////////////////////////
/// @brief World class is a container of 3D objects.
///
/// The representation of the simulated world is factored
/// into graphical and geodesic classes: Geo and World.
///
class World
{
private: typedef vector<shptr<Object> > Zombies;		///< a list would allocate too much

public:						World( void );
public:						~World();
public: void                Make( void );

// Interface:
public:	void				Attach( shptr<Object> object,
                                    const StateBits stateBits = STATE_BIT_DEFAULT,
                                    const RefPtr<osg::StateSet> stateSet = new osg::StateSet );
public:	void				Detach( shptr<Object> object );
public: void				OptimizeSceneGraph( void );  // to be called after loading objects
public: bool				IfFlat( void );
public: WorldVertex         GetPosition( void ) { return WorldVertex(0,0,0); }  // center
public: fp                  GetRadius( void );
public: bool                SetCompression( const bool enable );
public: fpx					GetCompressionScale( void );
public: void                RotateTangential( Object& object, const SphereVertex& sv );
public: const Matrix&		GetTangentMatrix( const SphereVertex& sv );
public: WorldVertex         ComputeUpVector( const SphereVertex& sv );
public: WorldVertex         ComputeUpVector( Object& object );
public: BoxVolume			GetVolume( void );

// Indirect way for Objects to delete themselves.
public:	 void				SpawnZombie( shptr<Object> obj );
private: void				KillZombies( void );

// Orientation:
public:  CLASS_CONST int ALT_UP_DIR   = 1;
public:  CLASS_CONST int ALT_DOWN_DIR = -1;
public:  bool               IfDownward( fp n ) { return n < 0; }
public:  bool               IfUpward( fp n )   { return n >= 0; }

private:
	/// @brief Timer-tick used to destroy zombie Objects.
	///
	class TickFunctor : public base::Timer::Functor
	{
	public:
		void operator()( const Milliseconds millisecElapsed );
	};
	friend class TickFunctor;

private:
  //shptr<SceneGraph>		mSceneGraph;				///< Window now contains SceneGraph
	RefPtr<GroupNode>		mRootGroup;
	shptr<StateSorter>		mStateSorter;				///< state-sorting
	Zombies					mZombies;					///< indirect way for 3D Objects to delete themselves
};

} // namespace world

#endif // WORLD_WORLD_HH
